import { EVMContractAddress, EVMAccountAddress, BlockchainCommonErrors } from "@snickerdoodlelabs/objects";
import { ethers } from "ethers";
import { ResultAsync } from "neverthrow";
import { IBaseContract, WrappedTransactionResponse } from "../interfaces/index.js";
import { ContractOverrides } from "../interfaces/objects/index.js";
export declare abstract class BaseContract<TContractSpecificError> implements IBaseContract {
    protected providerOrSigner: ethers.providers.Provider | ethers.providers.JsonRpcSigner | ethers.Wallet;
    protected contractAddress: EVMContractAddress;
    protected abi: ethers.ContractInterface;
    protected contract: ethers.Contract;
    protected contractAbi: ethers.ContractInterface;
    constructor(providerOrSigner: ethers.providers.Provider | ethers.providers.JsonRpcSigner | ethers.Wallet, contractAddress: EVMContractAddress, abi: ethers.ContractInterface);
    getContractAddress(): EVMContractAddress;
    getContract(): ethers.Contract;
    protected generateError(error: any, errorMessage: string): TContractSpecificError | BlockchainCommonErrors;
    protected abstract generateContractSpecificError(msg: string, reason: string | undefined, e: unknown): TContractSpecificError;
    protected writeToContract(functionName: string, functionParams: any[], overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | TContractSpecificError>;
    static buildWrappedTransactionResponse(transactionResponse: ethers.providers.TransactionResponse, contractAddress: EVMContractAddress, signerAddress: EVMAccountAddress, functionName: string, functionParams: any[], contractAbi: any): WrappedTransactionResponse;
    static extractFunctionAbi(functionName: string, contractAbi: any): string;
}
//# sourceMappingURL=BaseContract.d.ts.map