"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var BaseContract_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseContract = void 0;
const objects_1 = require("@snickerdoodlelabs/objects");
const ethers_1 = require("ethers");
const inversify_1 = require("inversify");
const neverthrow_1 = require("neverthrow");
const index_js_1 = require("../interfaces/index.js");
let BaseContract = BaseContract_1 = class BaseContract {
    constructor(providerOrSigner, contractAddress, abi) {
        this.providerOrSigner = providerOrSigner;
        this.contractAddress = contractAddress;
        this.abi = abi;
        this.contract = new ethers_1.ethers.Contract(contractAddress, abi, providerOrSigner);
        this.contractAbi = abi;
    }
    getContractAddress() {
        return this.contractAddress;
    }
    getContract() {
        return this.contract;
    }
    generateError(error, errorMessage) {
        return objects_1.BlockchainErrorMapper.buildBlockchainError(error, (msg, reason, err) => this.generateContractSpecificError(errorMessage || msg, reason, err));
    }
    // Takes the contract's function name and params, submits the transaction and returns a WrappedTransactionResponse
    writeToContract(functionName, functionParams, overrides) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract[functionName](...functionParams, {
            ...overrides,
        }), (e) => {
            return e;
        })
            .map((transactionResponse) => {
            var _a;
            return BaseContract_1.buildWrappedTransactionResponse(transactionResponse, (0, objects_1.EVMContractAddress)(this.contract.address), (0, objects_1.EVMAccountAddress)((_a = this.providerOrSigner) === null || _a === void 0 ? void 0 : _a.address), functionName, functionParams, this.contractAbi);
        })
            .mapErr((e) => {
            return objects_1.BlockchainErrorMapper.buildBlockchainError(e, (msg, reason, e) => this.generateContractSpecificError(msg, reason, e));
        });
    }
    // Function to return the correct error type based on mapping of error message
    static buildWrappedTransactionResponse(transactionResponse, contractAddress, signerAddress, functionName, functionParams, contractAbi) {
        return new index_js_1.WrappedTransactionResponse(transactionResponse, contractAddress, signerAddress, functionName, JSON.stringify(functionParams || []), BaseContract_1.extractFunctionAbi(functionName, contractAbi));
    }
    static extractFunctionAbi(functionName, contractAbi) {
        if (contractAbi == null || contractAbi.abi instanceof Array === false) {
            return "";
        }
        for (const item of contractAbi.abi) {
            if (item.type === "function" && item.name === functionName) {
                return item;
            }
        }
        return "";
    }
};
BaseContract = BaseContract_1 = __decorate([
    (0, inversify_1.injectable)()
], BaseContract);
exports.BaseContract = BaseContract;
//# sourceMappingURL=BaseContract.js.map