import { BaseURI, BigNumberString, ConsentFactoryContractError, ConsentName, EVMAccountAddress, EVMContractAddress, MarketplaceListing, MarketplaceTag, BlockchainCommonErrors, TransactionResponseError } from "@snickerdoodlelabs/objects";
import { ethers } from "ethers";
import { ResultAsync } from "neverthrow";
import { BaseContract } from "../implementations/BaseContract.js";
import { IConsentFactoryContract } from "../interfaces/IConsentFactoryContract.js";
import { ConsentRoles, ContractOverrides, WrappedTransactionResponse } from "../interfaces/objects/index.js";
export declare class ConsentFactoryContract extends BaseContract<ConsentFactoryContractError> implements IConsentFactoryContract {
    protected providerOrSigner: ethers.providers.Provider | ethers.providers.JsonRpcSigner | ethers.Wallet;
    protected contractAddress: EVMContractAddress;
    constructor(providerOrSigner: ethers.providers.Provider | ethers.providers.JsonRpcSigner | ethers.Wallet, contractAddress: EVMContractAddress);
    getContractAddress(): EVMContractAddress;
    createConsent(ownerAddress: EVMAccountAddress, baseUri: BaseURI, name: ConsentName, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ConsentFactoryContractError>;
    estimateGasToCreateConsent(ownerAddress: EVMAccountAddress, baseUri: BaseURI, name: ConsentName): ResultAsync<ethers.BigNumber, ConsentFactoryContractError | BlockchainCommonErrors>;
    getUserDeployedConsentsCount(ownerAddress: EVMAccountAddress): ResultAsync<number, ConsentFactoryContractError | BlockchainCommonErrors>;
    getUserDeployedConsentsByIndex(ownerAddress: EVMAccountAddress, startingIndex: number, endingIndex: number): ResultAsync<EVMContractAddress[], ConsentFactoryContractError | BlockchainCommonErrors>;
    getUserDeployedConsents(ownerAddress: EVMAccountAddress): ResultAsync<EVMContractAddress[], ConsentFactoryContractError | BlockchainCommonErrors>;
    getUserRoleAddressesCount(ownerAddress: EVMAccountAddress, role: ConsentRoles): ResultAsync<number, ConsentFactoryContractError | BlockchainCommonErrors>;
    getUserRoleAddressesCountByIndex(ownerAddress: EVMAccountAddress, role: ConsentRoles, startingIndex: number, endingIndex: number): ResultAsync<EVMContractAddress[], ConsentFactoryContractError | BlockchainCommonErrors>;
    getDeployedConsents(): ResultAsync<EVMContractAddress[], ConsentFactoryContractError | BlockchainCommonErrors>;
    getMaxTagsPerListing(): ResultAsync<number, ConsentFactoryContractError | BlockchainCommonErrors>;
    getListingDuration(): ResultAsync<number, ConsentFactoryContractError | BlockchainCommonErrors>;
    setListingDuration(listingDuration: number, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ConsentFactoryContractError>;
    setMaxTagsPerListing(maxTagsPerListing: number, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ConsentFactoryContractError>;
    adminRemoveListing(tag: MarketplaceTag, removedSlot: BigNumberString, overrides?: ContractOverrides): ResultAsync<WrappedTransactionResponse, BlockchainCommonErrors | ConsentFactoryContractError>;
    getListingDetail(tag: MarketplaceTag, slot: BigNumberString): ResultAsync<MarketplaceListing, ConsentFactoryContractError | BlockchainCommonErrors>;
    getListingsForward(tag: MarketplaceTag, startingSlot: BigNumberString, numberOfSlots: number, removeExpired: boolean): ResultAsync<MarketplaceListing[], ConsentFactoryContractError | BlockchainCommonErrors>;
    getListingsBackward(tag: MarketplaceTag, startingSlot: BigNumberString, numberOfSlots: number, removeExpired: boolean): ResultAsync<MarketplaceListing[], ConsentFactoryContractError | BlockchainCommonErrors>;
    getTagTotal(tag: MarketplaceTag): ResultAsync<number, ConsentFactoryContractError | BlockchainCommonErrors>;
    getListingsByTag(tag: MarketplaceTag, removeExpired: boolean): ResultAsync<MarketplaceListing[], ConsentFactoryContractError | BlockchainCommonErrors>;
    getAddressOfConsentCreated(txRes: WrappedTransactionResponse): ResultAsync<EVMContractAddress, TransactionResponseError>;
    protected generateContractSpecificError(msg: string, reason: string | undefined, e: unknown): ConsentFactoryContractError;
}
//# sourceMappingURL=ConsentFactoryContract.d.ts.map