"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CrumbsContract = void 0;
const objects_1 = require("@snickerdoodlelabs/objects");
const inversify_1 = require("inversify");
const neverthrow_1 = require("neverthrow");
const BaseContract_js_1 = require("../implementations/BaseContract.js");
const index_js_1 = require("../interfaces/objects/index.js");
let CrumbsContract = class CrumbsContract extends BaseContract_js_1.BaseContract {
    constructor(providerOrSigner, contractAddress) {
        super(providerOrSigner, contractAddress, index_js_1.ContractsAbis.CrumbsAbi.abi);
        this.providerOrSigner = providerOrSigner;
        this.contractAddress = contractAddress;
    }
    getContractAddress() {
        return this.contractAddress;
    }
    addressToCrumbId(accountAddress) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.addressToCrumbId(accountAddress), (e) => {
            return this.generateError(e, "Unable to call addressToCrumbId()");
        }).map((tokenId) => {
            // The contract returns 0 for an address that does not have a Crumb Id
            // Handle by returning null
            if (tokenId.eq(0)) {
                return null;
            }
            return (0, objects_1.TokenId)(tokenId.toBigInt());
        });
    }
    tokenURI(tokenId) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.tokenURI(tokenId), (e) => {
            return this.generateError(e, "Unable to call tokenURI()");
        }).orElse((error) => {
            // The contract reverts with this message if tokenId does not exist
            if (error.reason === "ERC721: operator query for nonexistent token") {
                return (0, neverthrow_1.okAsync)(null);
            }
            return (0, neverthrow_1.errAsync)(error);
        });
    }
    createCrumb(crumbId, tokenUri, overrides) {
        return this.writeToContract("createCrumb", [crumbId, tokenUri], overrides);
    }
    encodeCreateCrumb(crumbId, crumbContent) {
        return (0, objects_1.HexString)(this.contract.interface.encodeFunctionData("createCrumb", [
            crumbId,
            crumbContent,
        ]));
    }
    burnCrumb(crumbId, overrides) {
        return this.writeToContract("createCrumb", [crumbId], overrides);
    }
    encodeBurnCrumb(crumbId) {
        return (0, objects_1.HexString)(this.contract.interface.encodeFunctionData("burnCrumb", [crumbId]));
    }
    updateTokenURI(crumbId, tokenURI, overrides) {
        return this.writeToContract("updateTokenURI", [crumbId, tokenURI], overrides);
    }
    generateContractSpecificError(msg, reason, e) {
        return new objects_1.CrumbsContractError(msg, reason, e);
    }
};
CrumbsContract = __decorate([
    (0, inversify_1.injectable)()
], CrumbsContract);
exports.CrumbsContract = CrumbsContract;
//# sourceMappingURL=CrumbsContract.js.map