"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ERC721RewardContract = void 0;
const objects_1 = require("@snickerdoodlelabs/objects");
const inversify_1 = require("inversify");
const neverthrow_1 = require("neverthrow");
const neverthrow_result_utils_1 = require("neverthrow-result-utils");
const BaseContract_js_1 = require("../implementations/BaseContract.js");
const ERewardRoles_js_1 = require("../interfaces/enums/ERewardRoles.js");
const index_js_1 = require("../interfaces/objects/index.js");
let ERC721RewardContract = class ERC721RewardContract extends BaseContract_js_1.BaseContract {
    constructor(providerOrSigner, contractAddress) {
        super(providerOrSigner, contractAddress, index_js_1.ContractsAbis.ERC721Reward.abi);
        this.providerOrSigner = providerOrSigner;
        this.contractAddress = contractAddress;
        this.filters = {
            Transfer: (fromAddress, toAddress) => {
                return this.contract.filters.Transfer(fromAddress, toAddress);
            },
        };
    }
    getContractAddress() {
        return this.contractAddress;
    }
    getOwner() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMember(ERewardRoles_js_1.ERewardRoles.DEFAULT_ADMIN_ROLE, 0), (e) => {
            return this.generateError(e, "Unable to call getRoleMember()");
        });
    }
    // Returns an array of address that has the DEFAULT_ADMIN_ROLE on the Reward contract
    // Note that the address on index 0 is the contract owner
    getDefaultAdminRoleMembers() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMemberCount(ERewardRoles_js_1.ERewardRoles.DEFAULT_ADMIN_ROLE), (e) => {
            return this.generateError(e, "Unable to call getDefaultAdminRoleMembers()");
        }).andThen((memberCount) => {
            // First get an array of index values so that it can be used with ResultUtils.combine
            const memberIndexArray = [];
            for (let i = 0; i < memberCount.toNumber(); i++) {
                memberIndexArray.push(i);
            }
            // Map through each index value and get its role member address
            return neverthrow_result_utils_1.ResultUtils.combine(memberIndexArray.map((index) => {
                return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMember(ERewardRoles_js_1.ERewardRoles.DEFAULT_ADMIN_ROLE, index), (e) => {
                    return this.generateError(e, "Unable to call getRoleMember()");
                });
            }));
        });
    }
    getMinterRoleMembers() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMemberCount(ERewardRoles_js_1.ERewardRoles.MINTER_ROLE), (e) => {
            return this.generateError(e, "Unable to call getSignerRoleMembers()");
        }).andThen((memberCount) => {
            // First get an array of index values so that it can be used with ResultUtils.combine
            const memberIndexArray = [];
            for (let i = 0; i < memberCount.toNumber(); i++) {
                memberIndexArray.push(i);
            }
            // Map through each index value and get its role member address
            return neverthrow_result_utils_1.ResultUtils.combine(memberIndexArray.map((index) => {
                return neverthrow_1.ResultAsync.fromPromise(this.contract.getRoleMember(ERewardRoles_js_1.ERewardRoles.MINTER_ROLE, index), (e) => {
                    return this.generateError(e, "Unable to call getRoleMember() for MINTER_ROLE");
                });
            }));
        });
    }
    baseURI() {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.baseURI(), (e) => {
            return this.generateError(e, "Unable to call baseURI()");
        });
    }
    setBaseURI(baseUri, overrides) {
        return this.writeToContract("setBaseURI", [baseUri, overrides]);
    }
    balanceOf(address) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.balanceOf(address), (e) => {
            return this.generateError(e, "Unable to call balanceOf()");
        }).map((numberOfTokens) => {
            return numberOfTokens.toNumber();
        });
    }
    ownerOf(tokenId) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.ownerOf(tokenId), (e) => {
            return this.generateError(e, "Unable to call ownerOf()");
        });
    }
    tokenURI(tokenId) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.tokenURI(tokenId), (e) => {
            return this.generateError(e, "Unable to call tokenURI()");
        }).orElse((error) => {
            // The contract reverts with this message if tokenId does not exist
            if (error.reason === "ERC721: operator query for nonexistent token") {
                return (0, neverthrow_1.okAsync)(null);
            }
            return (0, neverthrow_1.errAsync)(error);
        });
    }
    hasRole(role, address) {
        return neverthrow_1.ResultAsync.fromPromise(this.contract.hasRole(ERewardRoles_js_1.ERewardRoles[role], address), (e) => {
            return this.generateError(e, "Unable to call hasRole()");
        });
    }
    grantRole(role, address, overrides) {
        return this.writeToContract("grantRole", [ERewardRoles_js_1.ERewardRoles[role], address], overrides);
    }
    revokeRole(role, address, overrides) {
        return this.writeToContract("revokeRole", [ERewardRoles_js_1.ERewardRoles[role], address], overrides);
    }
    renounceRole(role, address, overrides) {
        return this.writeToContract("renounceRole", [ERewardRoles_js_1.ERewardRoles[role], address], overrides);
    }
    generateContractSpecificError(msg, reason, e) {
        return new objects_1.ERC721RewardContractError(msg, reason, e);
    }
};
ERC721RewardContract = __decorate([
    (0, inversify_1.injectable)()
], ERC721RewardContract);
exports.ERC721RewardContract = ERC721RewardContract;
//# sourceMappingURL=ERC721RewardContract.js.map